<?php

/**
 * @file
 * The theme system, which controls the output of Date Picker Formatter.
 */

/**
 * Implements hook_theme().
 */
function date_picker_formatter_theme_theme() {
  return array(
    'date_picker_formatter' => array(
      'file' => 'date_picker_formatter.theme.inc',
      'variables' => array('entity' => NULL, 'field' => NULL, 'items' => NULL, 'display' => NULL),
    ),
    'date_picker_formatter_date_cell' => array(
      'file' => 'date_picker_formatter.theme.inc',
      'variables' => array('infos' => NULL),
    ),
    'date_picker_formatter_time_cell' => array(
      'file' => 'date_picker_formatter.theme.inc',
      'variables' => array('infos' => NULL),
    ),
    'date_picker_formatter_user_cell' => array(
      'file' => 'date_picker_formatter.theme.inc',
      'variables' => array('uid' => NULL),
    ),
    'date_picker_formatter_vote_cell' => array(
      'file' => 'date_picker_formatter.theme.inc',
      'variables' => array('vote' => NULL),
    ),
    'date_picker_formatter_stats_cell' => array(
      'file' => 'date_picker_formatter.theme.inc',
      'variables' => array('vote' => NULL, 'options' => NULL),
    ),
    'date_picker_formatter_stats_count' => array(
      'file' => 'date_picker_formatter.theme.inc',
      'variables' => array('count' => NULL),
    ),
    'date_picker_formatter_element_form' => array(
      'file' => 'date_picker_formatter.theme.inc',
      'render element' => 'form',
    ),
  );
}

/**
 * Theme the date element.
 */
function theme_date_picker_formatter($variables) {
  $form = drupal_get_form('date_picker_formatter_element_form', $variables);
  return drupal_render($form);
}

/**
 * Theme a date cell.
 */
function theme_date_picker_formatter_date_cell($variables) {
  extract($variables['info']);
  $output = '';

  if (isset($date2) && $date != $date2) {
    $output .= '<span class="date first">'. $date . ($time ? ' <span class="time">'. $time .'</span>' : '') .'</span> ';
    $output .= '<span class="separator">-</span> ';
    $output .= '<span class="date second">'. $date2 . (@$time2 ? ' <span class="time">'. $time2 .'</span>' : '') .'</span>';
  }
  else {
    $output .= '<span class="date">'. $date .'</span>';
  }

  return $output;
}

/**
 * Theme a time cell.
 */
function theme_date_picker_formatter_time_cell($variables) {
  extract($variables['info']);
  $output = '';

  if (isset($time2) && $time != $time2) {
    $output .= '<span class="time first">'. $time .'</span> ';
    $output .= '<span class="separator">-</span> ';
    $output .= '<span class="time second">'. $time2 .'</span>';
  }
  else {
    $output .= '<span class="time">'. $time .'</span>';
  }

  return $output;
}

/**
 * Theme a user cell.
 */
function theme_date_picker_formatter_user_cell($variables) {
  $uid = $variables['uid'];
  global $user;
  // TODO Convert "user_load" to "user_load_multiple" if "$uid" is other than a uid.
  // To return a single user object, wrap "user_load_multiple" with "array_shift" or equivalent.
  // Example: array_shift(user_load_multiple(array(), $uid))
  $account = user_load($uid);
  $output = '';
  if (variable_get('date_picker_formatter_display_user_picture', 1)) {
    $output .= theme('user_picture', array('account' => $account));
  }

  // Use the default theme_username to show voting user
  $output .= theme('username', array('account' => $account));

  if ($account->uid == $user->uid) {
    $output .= ' (' . t('You') . ')';
  }
  return $output;
}

/**
 * Theme a vote cell.
 */
function theme_date_picker_formatter_vote_cell($variables) {
  return '<span class="vote vote-' . $variables['vote']['value'] . '">' . $variables['vote']['label'] . '</span>';
}

/**
 * Theme a voting stats cell.
 */
function theme_date_picker_formatter_stats_cell($variables) {
  $vote = $variables['vote'];
  $options = $variables['options'];

  $items = array();
  foreach ($vote as $id => $count) {
    $items[] =  format_plural($count, "@label: 1 vote", "@label: @count votes", array('@label' => $options[$id]));
  }

  return theme('item_list', array('items' => $items));
}

/**
 * Theme the voting users count.
 */
function theme_date_picker_formatter_stats_count($variables) {
  return format_plural($variables['count'], '1 votes', '@count votes');
}

/**
 * Theme the date picker form.
 */
function theme_date_picker_formatter_element_form($variables) {
  global $user;
  $form = &$variables['form'];

  // Don't display form if no #infos provided.
  if (empty($form['dates']['#infos'])) {
    return '';
  }

  drupal_add_css(drupal_get_path('module', 'date_picker_formatter') . '/date_picker_formatter.css');

  $output = '';
  $rows = array();

  $header = array();
  $header[] = array(
    'data' => t('Users'),
    'rowspan' => 2,
    'class' => array('user-header'),
  );

  $second_row = array(
    array(
      'data' => '<!-- top left corner time -->',
      'class' => array('header'),
    ),
  );

  $infos = $form['dates']['#infos'];
  $keys = array_keys($infos);
  $prev_key = array_shift($keys);
  $prev_date = NULL;

  foreach ($infos as $key => $info) {
    extract($info);
    if ((isset($date2) && ($date != $date2 || $prev_date != $date2)) || !isset($date2)) {
      $header[$key] = array(
        'class' => array('date-header'),
        'data' => theme('date_picker_formatter_date_cell', array('info' => $info)),
      );
      $prev_key = $key;
    }
    else {
      $header[$prev_key]['colspan'] = (!isset($header[$prev_key]['colspan'])) ? 1 : $header[$prev_key]['colspan'];
      $header[$prev_key]['colspan']++;
    }
    $prev_date = $date;
    $second_row[$key] = array(
      'class' => array('time'),
      'data' => theme('date_picker_formatter_time_cell', array('info' => $info)),
    );
  }

  $rows['times'] = array(
    'class' => array('time', 'header'),
    'data' => $second_row,
  );

  foreach (element_children($form['dates']) as $uid) {
    $row = array(
      'user' => array(
        'class' => array('user', 'header'),
        'data' => theme('date_picker_formatter_user_cell', array('uid' => $uid)),
      ),
    );
    foreach (element_children($form['dates'][$uid]) as $key) {
      unset($form['dates'][$uid][$key]['#title']);
      $row[$key] = array(
        'class' => array('answer', @$form['dates'][$uid][$key]['#attributes']['class']),
        'data' => drupal_render($form['dates'][$uid][$key]),
      );
    }
    $rows[$uid] = array(
      'class' => array('result'),
      'data' => $row,
    );
  }

  // Add the "Submit" button to the current user row.
  if (isset($rows[$user->uid])) {
    $rows['current_user'] = $rows[$user->uid];
    unset($rows[$user->uid]);
    $rows['current_user']['data']['user']['data'] .= ' ' . drupal_render($form['submit']);

    // Adding CSS for current-user row theming.
    $rows['current_user']['class'][] = 'current-user';
  }

  // Adding stats.
  $stats = date_picker_formatter_get_stats($form['#votes'], array_keys($infos), $form['#control'], $form['#options']);

  $stats_row = array(
    'class' => array('stats'),
    'data' => array(
      'count' => theme('date_picker_formatter_stats_count', array('count' => $stats['voting_users'])),
    ),
  );

  foreach ($stats['votes'] as $vote) {
    $stats_row['data'][] = array(
      'class' => array('answer-stats', _date_picker_formatter_get_class($vote)),
      'data' => theme('date_picker_formatter_stats_cell', array('vote' => $vote, 'options' => $form['#options'])),
    );
  }

  $rows[] = $stats_row;

  $attributes = array(
    'id' => 'date-picker-formatter-table-' . $form['#entity_id'] . '-' . str_replace('_', '-', $form['#field_name']),
    'class' => array('date-picker-formatter-table'),
  );

  $output .= theme('table', array('header' => $header, 'rows' => $rows, 'attributes' => $attributes));
  $output .= drupal_render_children($form);

  return $output;
}
